<?php

/**
 * @file
 * Page callbacks for swim admin pages.
 */

/**
 * Admin structure page callback, lists all swim entities.
 */
function swim_admin_structure_page() {
  $item = menu_get_item();
  $content = system_admin_menu_block($item);
  return theme('node_add_list', array('content' => $content));
}


/**
 * Swim settings form.
 */
function swim_admin_settings_form() {
  global $_SWIM_COURSES;
  $form = array();
  $form['swim_default_course'] = array(
    '#type' => 'select',
    '#title' => t('Default course'),
    '#default_value' => variable_get('swim_default_course', 'SCY'),
    '#options' => $_SWIM_COURSES,
    '#description' => t('Default swimming course for events and display.'),
  );

  $form['swim_ageup_date'] = array(
    '#type' => 'textfield',
    '#title' => t('Age-up date'),
    '#description' => t('The date format is MM/DD (Season year is added by the system).'),
    '#default_value' => variable_get('swim_ageup_date', '06/15'),
  );

  // Add system_settings stuff.
  $form = system_settings_form($form);

  // Add extra submit handler for rebuilding menus.
  $form['#submit'][] = 'swim_admin_settings_form_submit';

  return $form;
}


/*
 * Validate the swim settings form.
 */
function swim_admin_settings_form_validate($form, &$form_state) {
  // swim_ageup_date must be a "MM/DD".
  if (strtotime($form_state['values']['swim_ageup_date'] . '/2012') === FALSE) {
    form_set_error('swim_ageup_date', t('Age-up date must be a valid date'));
  }
}


/**
 * Extra submit handler for swim_admin_settings_form().
 * Need to rebuild menus so dynamic tabs will show/not show correctly.
 */
function swim_admin_settings_form_submit() {
  menu_rebuild();
}

/**
 * Menu callback: swim meet administration.
 */
function swim_admin_meet($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return swim_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['meets']));
  }
  $form['admin'] = swim_admin_meet_items();

  return $form;
}


function swim_admin_meet_items() {
  global $_SWIM_MEET_TYPE;
  $header = array(
    array('data' => t('Season'), 'field' => 'season_name'),
    array('data' => t('Meet'), 'field' => 'meet_name'),
    array('data' => t('Date'), 'field' => 'start_date', 'sort' => 'asc'),
    t('Operations'),
  );
  $rows = array();

  $rows = array();
  $items = swim_get_meets(NULL, array('orderByHeader' => $header));
  foreach ($items as $meet) {
    $row = array('data' => array());
    $row['data'][] = array('data' => $meet->season_name);
    $row['data'][] = array('data' => $meet->meet_name . ' (' . $_SWIM_MEET_TYPE[$meet->meet_type] . ')');
    $row['data'][] = array('data' => format_date($meet->start_date, 'custom', 'm/d/Y'));
    $row['data'][] = array('data' => l(t('edit'), 'swim/meet/' . $meet->mid . '/edit') . '&nbsp;&nbsp;' . l(t('delete'), 'swim/meet/' . $meet->mid . '/delete', array('query' => array('destination' => 'admin/structure/swim/meet'))));
    $rows[] = $row;
  }

  $build['add_new'] = array(
    '#markup' => l(t('+ Add meet'), 'swim/meet/add'),
    '#weight' => -1,
  );

  $build['swim_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No swim meets available.'),
  );

  return $build;
}


/**
 * Menu callback: swim season administration.
 */
function swim_admin_season($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return swim_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['seasons']));
  }
  $form['admin'] = swim_admin_season_items();

  return $form;
}


function swim_admin_season_items() {
  $header = array(
    array('data' => t('Name'), 'field' => 'season_name'),
    array('data' => t('Start date'), 'field' => 'start_date'),
    array('data' => t('End date'), 'field' => 'end_date'),
    array('data' => t('Inactive'), 'field' => 'inactive'),
    t('Operations'),
  );
  $rows = array();
  $items = swim_get_seasons(NULL, array('orderByHeader' => $header));
  foreach ($items as $season) {
    $season->start_date = isset($season->start_date) ? format_date($season->start_date, 'custom', 'Y-m-d') : '';
    $season->end_date = isset($season->end_date) ? format_date($season->end_date, 'custom', 'Y-m-d') : '';
    $row = array('data' => array());
    $row['data'][] = array('data' => $season->season_name);
    $row['data'][] = array('data' => $season->start_date);
    $row['data'][] = array('data' => $season->end_date);
    $row['data'][] = array('data' => $season->inactive);
    $row['data'][] = array('data' => l(t('edit'), 'swim/season/' . $season->sid . '/edit') . '&nbsp;&nbsp;' . l(t('delete'), 'swim/season/' . $season->sid . '/delete', array('query' => array('destination' => 'admin/structure/swim/season'))));
    $rows[] = $row;
  }

  $build['add_new'] = array(
    '#markup' => l(t('+ Add season'), 'swim/season/add'),
    '#weight' => -1,
  );

  $build['swim_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No swim seasons available.'),
  );

  return $build;
}


/**
 * Menu callback: swim season administration.
 */
function swim_admin_age_group($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return swim_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['age_group']));
  }
  $form['admin'] = swim_admin_age_group_items();

  return $form;
}


function swim_admin_age_group_items() {
  $header = array(
    array('data' => t('Name'), 'field' => 'group_name'),
    array('data' => t('Minimum age'), 'field' => 'age_min'),
    array('data' => t('Maximum age'), 'field' => 'age_max', 'sort' => 'asc'),
    t('Operations'),
  );
  $rows = array();
  $items = swim_get_age_groups(NULL, array('orderByHeader' => $header));
  foreach ($items as $group) {
    $row = array('data' => array());
    $row['data'][] = array('data' => $group->group_name);
    $row['data'][] = array('data' => $group->age_min);
    $row['data'][] = array('data' => $group->age_max);
    $row['data'][] = array('data' => l(t('edit'), 'swim/age_group/' . $group->gid . '/edit') . '&nbsp;&nbsp;' . l(t('delete'), 'swim/age_group/' . $group->gid . '/delete', array('query' => array('destination' => 'admin/structure/swim/age_group'))));
    $rows[] = $row;
  }

  $build['add_new'] = array(
    '#markup' => l(t('+ Add age group'), 'swim/age_group/add'),
    '#weight' => -1,
  );

  $build['swim_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No swim age groups available.'),
  );

  return $build;
}


/**
 * Menu callback: swim meet administration.
 */
function swim_admin_athlete($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return swim_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['athletes']));
  }
  $form['admin'] = swim_admin_athlete_items();

  return $form;
}


function swim_admin_athlete_items() {
  global $_SWIM_SEXES;
  $header = array(
    array('data' => t('Name'), 'field' => 'swim_athlete_full_name', 'sort' => 'asc'),
    array('data' => t('Sex'), 'field' => 'sex'),
    array('data' => t('Birthdate'), 'field' => 'birthdate'),
    t('Age'),
    t('Operations'),
  );
  $rows = array();

  $rows = array();
  $items = swim_get_athletes(NULL, array('orderByHeader' => $header));
  foreach ($items as $athlete) {
    $age = time() - $athlete->birthdate;
    $age = floor($age / (60*60*24*364));
    $row = array('data' => array());
    $row['data'][] = array('data' => swim_athlete_format_full_name($athlete));
    $row['data'][] = array('data' => $athlete->sex);
    $row['data'][] = array('data' => format_date($athlete->birthdate, 'custom', 'm/d/Y'));
    $row['data'][] = array('data' => $age);
    $row['data'][] = array('data' => l(t('edit'), 'swim/athlete/' . $athlete->aid . '/edit') .
      '&nbsp;&nbsp;' .
      l(t('delete'), 'swim/athlete/' . $athlete->aid . '/delete', array('query' => array('destination' => 'admin/structure/swim/athlete')))
    );
    $rows[] = $row;
  }

  $build['add_new'] = array(
    '#markup' => l(t('+ Add athlete'), 'swim/athlete/add'),
    '#weight' => -1,
  );

  $build['swim_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No swim athletes available.'),
  );

  return $build;
}


/**
 * Menu callback for administering swim events.
 */
function swim_admin_event($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return swim_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['events']));
  }
  $form['admin'] = swim_admin_event_items();

  return $form;
}


function swim_admin_event_items() {
  global $_SWIM_SEXES;
  $header = array(
    array('data' => t('Event'), 'field' => 'swim_event_full_name', 'sort' => 'asc'),
    t('I/R'),
    t('Operations'),
  );
  $rows = array();

  $rows = array();
  $items = swim_get_events(NULL, array('orderByHeader' => $header));
  foreach ($items as $event) {
    $event_full_name = swim_event_get_full_name($event);
    $row = array('data' => array());
    $row['data'][] = array('data' => $event_full_name);
    $row['data'][] = array('data' => $event->i_r);
    $row['data'][] = array('data' => l(t('edit'), 'swim/event/' . $event->eid . '/edit') . '&nbsp;&nbsp;' . l(t('delete'), 'swim/event/' . $event->eid . '/delete', array('query' => array('destination' => 'admin/structure/swim/event'))));
    $rows[] = $row;
  }

  $build['add_new'] = array(
    '#markup' => l(t('+ Add event'), 'swim/event/add'),
    '#weight' => -1,
  );

  $build['swim_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No swim events available.'),
  );

  return $build;
}


/**
 * Menu callback for administering swim records.
 */
function swim_admin_record($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return swim_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['records']));
  }
  $form['admin'] = swim_admin_record_items();

  return $form;
}
function swim_admin_record_items() {
  global $_SWIM_SEXES;
  $header = array(
    array('data' => t('Record Name'), 'field' => 'record_name', 'sort' => 'asc'),
    array('data' => t('Event'), 'field' => 'swim_event_full_name'),
    t('Time'),
    t('Operations')
  );
  $rows = array();
  $items = swim_get_records(NULL, array('orderByHeader' => $header));
  foreach ($items as $record) {
    $row = array('data' => array());
    $row['data'][] = array('data' => $record->record_name);
    $row['data'][] = array('data' => swim_event_get_full_name($record));
    $row['data'][] = array('data' => swim_format_time($record->time));
    $row['data'][] = array('data' => l(t('edit'), 'swim/record/' . $record->rec_id . '/edit') . '&nbsp;&nbsp;' . l(t('delete'), 'swim/record/' . $record->rec_id . '/delete', array('query' => array('destination' => 'admin/structure/swim/record'))));
    $rows[] = $row;
  }

  $build['add_new'] = array(
    '#markup' => l(t('+ Add record'), 'swim/record/add'),
    '#weight' => -1,

  );
  $build['swim_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No swim records available.'),
  );

  return $build;
}


/**
 * Menu callback for administering swim results.
 */
function swim_admin_result($form, $form_state) {
  if (isset($form_state['values']['operation']) && $form_state['values']['operation'] == 'delete') {
    return swim_multiple_delete_confirm($form, $form_state, array_filter($form_state['values']['results']));
  }
  $form['admin'] = swim_admin_result_items();

  return $form;
}
function swim_admin_result_items() {
  global $_SWIM_SEXES;
  $header = array(
    array('data' => t('Meet Name'), 'field' => 'meet_name'),
    array('data' => t('Date'), 'field' => 'start_date', 'sort' => 'desc'),
    array('data' => t('Event'), 'field' => 'swim_event_full_name'),
    array('data' => t('Swimmer'), 'field' => 'swim_athlete_full_name'),
    t('Time'), t('Operations'));
  $rows = array();

  $items = swim_get_swims(NULL, array('orderByHeader' => $header));
  foreach ($items as $swim) {
    $row = array('data' => array());
    $row['data'][] = array('data' => $swim->meet_name);
    $row['data'][] = array('data' => format_date($swim->start_date, 'custom', 'm/d/Y'));
    $row['data'][] = array('data' => swim_event_get_full_name($swim));
    $row['data'][] = array('data' => swim_athlete_format_full_name($swim));
    $row['data'][] = array('data' => swim_format_time($swim->time));
    $row['data'][] = array('data' => l(t('edit'), 'swim/result/' . $swim->rid . '/edit') . '&nbsp;&nbsp;' . l(t('delete'), 'swim/result/' . $swim->rid . '/delete', array('query' => array('destination' => 'admin/structure/swim/record'))));
    $rows[] = $row;
  }

  $build['add_new'] = array(
    '#markup' => l(t('+ Add result'), 'swim/result/add'),
    '#weight' => -1,
  );
  $build['swim_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No swim results available.'),
  );

  return $build;
}


function swim_admin_content_import_sdif_page() {
  return drupal_get_form('import_sdif_form');
}

function import_sdif_form($form, &$form_state) {
  $form = array();

  if (isset($form_state['stage2'])) {

    $form['file_uri'] = array(
      '#type' => 'hidden',
      '#value' => $form_state['file_uri'],
    );

    $form['add_athletes'] = array(
      '#type' => 'checkbox',
      '#title' => t('Add new swimmers/results'),
      '#default_value' => 0,
      '#description' => t('Otherwise only matching swimmers with their results will be added.'),
    );

    $form['add_results_from_team'] = array(
      '#type' => 'fieldset',
      '#title' => t('Import from these teams'),
      '#description' => t('If you do not check any teams, no swimmers or results will be added.'),
      '#tree' => TRUE,
    );
    foreach ($form_state['teams'] as $team) {
      $team_name_label = $team['full_team_name'];
      if (!empty($team['abbreviated_team_name'])) {
        $team_name_label .= ' (' . $team['abbreviated_team_name'] . ')';
      }
      $form['add_results_from_team'][check_plain($team['full_team_name'])] = array(
        '#type' => 'checkbox',
        '#title' => check_plain($team_name_label),
        '#default_value' => 0
      );
    }

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Import'),
    );
  }
  else {
    $form['#attributes'] = array('enctype' => "multipart/form-data");
    $form['swim_sdif_import_file'] = array(
      '#type' => 'file',
      '#title' => t('SDIF file'),
      '#default_value' => '',
      '#size' => 64,
    );


    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Upload'),
    );
  }
  return $form;
}

function import_sdif_form_submit($form, &$form_state) {

  //drupal_set_message(print_r($form_state, TRUE));

  // Load athletes
  $athletes = swim_get_athletes(NULL, array('by_name' => 1));
  // Load events
  $events = swim_get_events_keyed_on_age_sex_distance_stroke();
  // Load meets
  $meets = swim_get_meets_keyed_on_name_start_date();

  $meet_rec = FALSE;
  $athlete_rec = FALSE;
  $event_rec = FALSE;
  if (isset($form_state['values']['file_uri'])) {
    // Second pass, actually load data.
    // Read file into data array.
    $data = file($form_state['values']['file_uri']);
    $skip = 1;
    foreach ($data as $line) {
      $rec = get_sdif_record($line);
      switch ($rec['rec']) {
        case "A0":
          // File Description Record
          break;
        case "B1":
          // Meet Record
          if (!isset($meets[$rec['meet_name'] . $rec['meet_start']])) {
            // Add meet.
            $meet = (object) array(
              'mid' => 0,
              'start_date' => get_time_from_date($rec['meet_start']),
              'end_date' => get_time_from_date($rec['meet_end']),
              'meet_name' => $rec['meet_name'],
              'season' => $rec['_season'],
            );

            // Save the new meet.
            swim_meet_save($meet);
            // Load meet into $meets.
            $meets[$rec['meet_name'] . $rec['meet_start']] = $meet;
          }
          $meet_rec = $meets[$rec['meet_name'] . $rec['meet_start']];
          break;
        case "B2":
          // Meet Host Record, B2
          break;
        case "C1":
          // Team Id Record, C1
          if ($form_state['values']['add_results_from_team'][check_plain($rec['full_team_name'])] == 1) {
            $skip = 0;
          }
          else {
            $skip = 1;
          }
          break;
        case "C2":
          // Team Entry Record
          break;
        case "D0":
          if ($skip == 1) {
            continue;
          }
          if ($form_state['values']['add_athletes'] == 0 && !isset($athletes[$rec['_swimmer_name_key']])) {
            // Skip if we are not adding athletes and this one doesn't match an existing swimmer.
            continue;
          }
          elseif ($form_state['values']['add_athletes'] == 1 && !isset($athletes[$rec['_swimmer_name_key']])) {
            // Add swimmer.
            $athlete = (object) array(
              'aid' => 0,
              'uid' => NULL,
              'f_name' => isset($rec['_swimmer_fname']) ? $rec['_swimmer_fname'] : '',
              'm_name' => isset($rec['_swimmer_mname']) ? $rec['_swimmer_mname'] : '',
              'l_name' => isset($rec['_swimmer_lname']) ? $rec['_swimmer_lname'] : '',
              'sex' => $rec['swimmer_sex'],
              'birthdate' => get_time_from_date($rec['swimmer_bday']),
              'inactive' => 0,
            );
            // Save the new athlete.
            swim_athlete_save($athlete);
            // Load swimmer into $athletes.
            $athletes[$rec['_swimmer_name_key']] = $athlete;
          }
          $athlete_rec = $athletes[$rec['_swimmer_name_key']];

          if (!isset($events[$rec['_event_key']])) {
            $event = (object) array(
              'eid' => 0,
              'course' => $rec['_course'],
              'distance' => $rec['event_distance'],
              'stroke' => $rec['_stroke'],
              'i_r' => 'i',
              'sex' => $rec['event_sex'],
              'age_min' => is_numeric($rec['_age_min'])?$rec['_age_min']:0,
              'age_max' => is_numeric($rec['_age_max'])?$rec['_age_max']:199,
            );

            // Save the new event.
            swim_event_save($event);
            // Load event into $events.
            $events[$rec['_event_key']] = $event;
          }
          $event_rec = $events[$rec['_event_key']];

          $result = (object) array(
            'rid' => 0,
            'aid' => $athlete_rec->aid,
            'eid' => $event_rec->eid,
            'mid' => $meet_rec->mid,
            'time' => $rec['_event_final_time'],
            'time_code' => $rec['_time_code'],
            'place' => $rec['final_place'],
            'points' => $rec['final_points'],
          );

          // Save the new event.
          swim_result_save($result);

          //drupal_set_message('NYI: Should add result for ' . $rec['swimmer_name']);
          break;
        case "D3":
          // Individual Info Record
          break;
      }
    }
  }


  else {
    // First pass, read file and determine how many swimmers and
    // results will be added.
    $data = array();
    $validators = array('file_validate_extensions' => array('sd3 sdif cl2 txt'));
    if ($file = file_save_upload('swim_sdif_import_file', $validators, 'public://', FILE_EXISTS_RENAME)) {
      // Add file path for stage two.
      $form_state['file_uri'] = $file->uri;
      $form_state['filename'] = $file->filename;
      // Read file into data array.
      $data = file($file->uri);
      drupal_set_message(t('The SDIF file was successfully uploaded.'));
    }
    else {
      drupal_set_message(t('The attached file failed to upload.'), 'error');
      return;
    }

    $new_meets = 0;
    $new_results = 0;
    $matching_swimmers = array();
    $new_swimmers = array();
    $teams = array();
    foreach ($data as $line) {
      $rec = get_sdif_record($line);
      switch ($rec['rec']) {
        case "A0":
          // File Description Record, A0
          break;
        case "B1":
          // Meet Record, B1
          $new_meets++;
          break;
        case "B2":
          // Meet Host Record, B2
          break;
        case "C1":
          // Team Id Record, C1
          $teams[] = $rec;
          break;
        case "C2":
          // Team Entry Record, C2
          break;
        case "D0":
          // Individual Event Record, D0
          if (isset($athletes[$rec['_swimmer_name_key']])) {
            $matching_swimmers[$rec['_swimmer_name_key']] = 1;
          }

          if (!isset($athletes[$rec['_swimmer_name_key']]) && !isset($new_swimmers[$rec['_swimmer_name_key']])) {
            $new_swimmers[$rec['_swimmer_name_key']] = 1;
          }

          $new_results++;
          break;
        case "D3":
          // Individual Info Record
          break;

      }
    }

    $new_swimmer_count = count($new_swimmers);
    $matching_swimmer_count = count($matching_swimmers);
    $team_count = count($teams);
    drupal_set_message(format_plural($new_meets, "1 meet found.", "$new_meets meets found."));
    drupal_set_message(format_plural($team_count, "1 team found.", "$team_count teams found."));
    drupal_set_message(format_plural($new_swimmer_count, "1 new swimmer found.", "$new_swimmer_count new swimmers found."));
    drupal_set_message(format_plural($matching_swimmer_count, "1 matching swimmer found.", "$matching_swimmer_count matching swimmers found."));
    drupal_set_message(format_plural($new_results, "1 result found.", "$new_results results found."));

    $form_state['teams'] = $teams;

    $form_state['stage2'] = TRUE;
    $form_state['rebuild'] = TRUE;
  }
}


function get_stroke_from_code($sdif_code) {
  switch ($sdif_code) {
    case 1:
      return 'FR';
    case 2:
      return 'BK';
    case 3:
      return 'BR';
    case 4:
      return 'FL';
    case 5:
      return 'IM';
    case 6:
      return 'FR-relay';
    case 7:
      return 'Med-relay';
  }
}



/*
* COURSE Code 013   Course/Status code
*       Please note that there are alternatives for the three types
*       of pools.  The alpha characters make the file more readable.
*       Either may be used.
*       1 or S   Short Course Meters
*       2 or Y   Short Course Yards
*       3 or L   Long Course Meters
*       X        Disqualified
*/
function get_course_from_code($sdif_code) {
  switch ($sdif_code) {
    case 1:
      return 'SCM';
    case 2:
      return 'SCY';
    case 3:
      return 'LCM';
    case 'S':
      return 'SCM';
    case 'Y':
      return 'SCY';
    case 'L':
      return 'LCM';
  }
}


function get_time_from_date($date_str) {
  $m = substr($date_str, 0, 2);
  $d = substr($date_str, 2, 2);
  $y = substr($date_str, 4, 4);
  return strtotime("$m/$d/$y");
}


function get_sdif_record($line=NULL) {
  if (empty($line)) {
    return FALSE;
  }
  $rec = array('rec' => substr($line, 0, 2));
  switch ($rec['rec']) {
    case "A0":
      // File Description Record, A0
      return $rec;
    case "B1":
      // Meet Record, B1
      $rec['meet_name'] = trim(substr($line, 11, 30));
      $rec['meet_start'] = trim(substr($line, 121, 8));
      $rec['meet_end'] = trim(substr($line, 129, 8));
      $rec['_season'] = substr($rec['meet_start'], 4, 4);
     return $rec;
    case "B2":
      // Meet Host Record
      return $rec;
    case "C1":
      // Team Id Record, C1
      $rec['full_team_name'] = trim(substr($line, 17, 30));
      $rec['abbreviated_team_name'] = trim(substr($line, 47, 16));
      return $rec;
    case "C2":
      // Team Entry Record
      return $rec;
    case "D0":
      // Individual Event Record, D0
      $rec['org'] = substr($line, 2, 1);
      $rec['swimmer_name'] = trim(substr($line, 11, 28));
      $rec['swimmer_id'] = trim(substr($line, 39, 12));
      $rec['swimmer_bday'] = substr($line, 55, 8);
      $rec['swimmer_age'] = trim(substr($line, 63, 2));
      $rec['swimmer_sex'] = substr($line, 65, 1);
      $rec['event_sex'] = substr($line, 66, 1);
      $rec['event_age'] = trim(substr($line, 76, 4));  // UN08, 18OV, 0810
      $rec['event_distance'] = trim(substr($line, 67, 4));
      $rec['stroke_code'] = substr($line, 71, 1);
      $rec['swim_date'] = substr($line, 80, 8);
      $rec['seed_time'] = trim(substr($line, 88, 8));
      $rec['seed_course'] = trim(substr($line, 96, 1));
      $rec['prelim_time'] = trim(substr($line, 97, 8));
      $rec['prelim_course'] = trim(substr($line, 105, 1));
      $rec['final_time'] = trim(substr($line, 115, 8));
      $rec['final_course'] = trim(substr($line, 123, 1));
      $rec['final_heat'] = trim(substr($line, 128, 2));
      $rec['final_lane'] = trim(substr($line, 130, 2));
      $rec['final_place'] = trim(substr($line, 135, 3));
      $rec['final_points'] = trim(substr($line, 138, 4));

      // Non-SDIF fields for internal use.
      // Uppercase the key, just to make sure.
      $rec['_swimmer_name_key'] = strtoupper($rec['swimmer_name']);
      $name = explode(' ', $rec['swimmer_name']);
      if (isset($name[0])) {
        // Remove trailing ,
        if (substr($name[0], strlen($name[0]) - 1, 1) == ',') {
          $name[0] = substr($name[0], 0, strlen($name[0]) - 1);
        }
        $rec['_swimmer_lname'] = $name[0];
      }
      if (isset($name[1])) {
        $rec['_swimmer_fname'] = $name[1];
      }
      if (isset($name[2])) {
        $rec['_swimmer_mname'] = $name[2];
      }
      $rec['_stroke'] = get_stroke_from_code($rec['stroke_code']);
      $rec['_course'] = get_course_from_code($rec['final_course']);
      $rec['_age_min'] = substr($rec['event_age'], 0, 2);
      $rec['_age_max'] = substr($rec['event_age'], 2, 2);
      $rec['_event_key'] = $rec['_age_min'] . $rec['_age_max'] . $rec['event_sex'] . $rec['event_distance'] . $rec['_stroke'];

      /*
       * All TIME fields MUST match one of the following formats:
       * 1) All blanks.
       * 2) Have the format mm:ss.ss where mm is the minutes value and
       *    ss.ss is the seconds value to hundredths.  The third byte must
       *    be a colon or a blank, and the sixth byte must be a period.
       *    If a blank appears anywhere in this format, everything to the
       *    left must also be blank.
       * 3) TIME Code 020, left-justified and blank-filled.
       *
       *  TIME Code 020     Time explanation code
       *         NT   No Time
       *         NS   No Swim (or No Show)
       *         DNF  Did Not Finish
       *         DQ   Disqualified
       *         SCR  Scratch
       */
      if (preg_match('/[A-Z]/', $rec['final_time'])) {
        $rec['_time_code'] = $rec['final_time'];
        $rec['_event_final_time'] = NULL;
      }
      else {
        $rec['_time_code'] = '';
        $rec['_event_final_time'] = swim_parse_time($rec['final_time']);
      }

      return $rec;
    case "D3":
      // Individual Info Record
      return $rec;
    default:
      return FALSE;
  }
}
