<?php
/**
 * @file
 * Page callbacks for swim pages.
 */

/*
 * Page callback for the swim dashboard.
 */
function swim_dashboard_page() {
  global $user;
  // Don't want a title on this page.
  drupal_set_title('');
  drupal_add_css(drupal_get_path('module', 'swim') . '/swim.css');

  $last_meet = swim_get_meets(NULL, array('last_meet' => TRUE));
  $next_meet = swim_get_meets(NULL, array('next_meet' => TRUE));
  $linked_swimmers = swim_get_athletes(array('status' => 1, 'linked_uid' => $user->uid), array());

  $variables = array(
    'last_meet' => $last_meet,
    'next_meet' => $next_meet,
    'linked_swimmers' => $linked_swimmers,
  );
  return theme('swim_dashboard', $variables);
}

/*
 * Theme function for the swim dashboard.
 */
function theme_swim_dashboard($variables) {
  $output = '';
  if (variable_get('swim_results', 0) == 1) {
    $output = theme('swim_dashboard_last_meet', $variables);
  }
  $output .= theme('swim_dashboard_next_meet', $variables);
  return $output;
}

/*
 * Theme function for the swim dashboard.
 */
function theme_swim_dashboard_last_meet($variables) {
  global $_SWIM_MEET_TYPE;
  $last_meet = $variables['last_meet'];
  $linked_swimmers = $variables['linked_swimmers'];

  if (!$last_meet) {
    return '';
  }
  $time = format_date($last_meet->start_date, 'custom', 'h:ia');
  $meet_start = '';
  if ($time == '12:00am') {
    $meet_start = format_date($last_meet->start_date, 'custom', 'D, M d');
  }
  else {
    $meet_start = format_date($last_meet->start_date, 'custom', 'D, M d  h:ia');
  }

  $output = '<div class="last-meet-block">';
  $output .= '<h2>' . t('Last Meet') . '</h2>';
  $output .= '<div>' . $last_meet->meet_name . ' (' . $_SWIM_MEET_TYPE[$last_meet->meet_type] . '): ' . $meet_start . '</div>';
  foreach ($linked_swimmers as $aid => $swimmer) {
    $full_name_link = swim_athlete_format_full_name($swimmer);
    if (variable_get('swim_results', 0) == 1) {
      $full_name_link = l($full_name_link, 'swim/meet_list/' . $last_meet->mid . '/' . $swimmer->aid);
    }
    $output .= '<div><span class="swimmer">' . $full_name_link . '</span>';
    $swims = swim_get_swims(array('mid' => $last_meet->mid, 'aid' => $swimmer->aid), array());
    foreach ($swims as $rid => $swim) {
      $output .= '<div class="result">' . swim_event_get_full_name($swim) . ': ' . swim_format_time($swim->time) . '</div>';
    }
    $output .= '</div>';
  }
  $output .= '</div>';
  return $output;
}

/*
 * Theme function for the swim dashboard.
 */
function theme_swim_dashboard_next_meet($variables) {
  global $_SWIM_MEET_TYPE;
  $next_meet = $variables['next_meet'];

  if ($next_meet == FALSE ) {
    return '';
  }
  $time = format_date($next_meet->start_date, 'custom', 'h:ia');
  $meet_start = '';
  if ($time == '12:00am') {
    $meet_start = format_date($next_meet->start_date, 'custom', 'D, M d');
  }
  else {
    $meet_start = format_date($next_meet->start_date, 'custom', 'D, M d  h:ia');
  }

  $output = '<div class="block">';
  $output .= '<h2>' . t('Next Meet') . '</h2>';
  $output .= '<div>' . $next_meet->meet_name . ' (' . $_SWIM_MEET_TYPE[$next_meet->meet_type] . '): ' . $meet_start . '</div>';
  $output .= '</div>';
  return $output;
}

/*
 * Page callback for a list of swim meets by season.
 */
function swim_dashboard_results_page() {
  // Don't want a title on this page.
  drupal_set_title('');
  // Don't want breadcrumbs on this page.
  drupal_set_breadcrumb(array());

  $variables = array('meets' => swim_get_meets(NULL, array('by_season' => 1)));
  return theme('swim_dashboard_results', $variables);
}

/*
 * Theme function for a list of swim meets by season.
 */
function theme_swim_dashboard_results($variables) {
  drupal_add_css(drupal_get_path('module', 'swim') . '/swim.css');

  $meets = $variables['meets'];
  $header = array(
    array('data' => t('Date')),
    array('data' => t('Name')),
  );
  $rows = array();
  $pager_total = count($meets);
  $page = pager_default_initialize($pager_total, 1, $element = 0);

  $query_page = -1;
  $caption = '';
  foreach ($meets as $season => $season_meets) {
    $query_page++;
    if ($query_page != $page) {
      continue;
    }
    $caption = $season . ' ' . t('Season');
    foreach ($season_meets as $mid => $meet) {
      $row = array('data' => array());
      $row['data'][] = format_date($meet->start_date, 'custom', 'Y-m-d');
      $row['data'][] = l($meet->meet_name, 'swim/meet_list/' . $meet->mid);
      $rows[] = $row;
    }

  }

  $output = '';
  if (user_access('create swim entries')) {
    $output .= '<h3>' . l(t('+ Add meet'), 'swim/meet/add') . '</h3>';
  }
  $output .= theme('table', array('caption' => $caption, 'header' => $header, 'rows' => $rows, 'sticky' => TRUE, 'attributes' => array('class' => array('meets'))));
  $output .= theme('pager', array('quantity' => $pager_total));
  return $output;
}

/*
 * List all meet results for a single meet.  Can also filter by swimmer.
 */
function swim_meet_page($meet=NULL, $athlete=NULL) {

  $swims = array();
  $header = array(
    array('data' => t('Swimmer'), 'field' => 'swim_athlete_full_name'),
    array('data' => t('Event'), 'field' => 'swim_event_full_name', 'sort' => 'asc'),
    array('data' => t('Time')),
    array('data' => t('Place')),
    array('data' => t('Points')),
    array('data' => t('Best / Time Code'))
  );

  if ($athlete === NULL) {
    $swims = swim_get_swims(array('mid' => $meet->mid), array('orderByHeader' => $header));
  }
  else {
    $swims = swim_get_swims(array('mid' => $meet->mid, 'aid' => $athlete->aid), array('orderByHeader' => $header));

    // Get rid of extra path element.
    $breadcrumb = drupal_get_breadcrumb();
    array_pop($breadcrumb);
    drupal_set_breadcrumb($breadcrumb);
  }

  $best_swims = swim_get_swims(array('best_swims' => 1));
  foreach ($swims as $rid => $swim) {
    // Handle personal bests.
    if (!isset($swim->pb)) {
      $swim->pb = 0;
    }
    foreach ($best_swims as $rid => $b_swim) {
      // Skip best swim if best swim meet date is after this meet.
      if ($b_swim->start_date > $swim->start_date) {
        continue;
      }
      // Skip best swim is for a different event.
      if ($b_swim->eid != $swim->eid) {
        continue;
      }
      // End loop if best swim meet date is before this meet.
      if ($b_swim->start_date < $swim->start_date) {
        break;
      }

      $swim->pb++;
      // Since we are only checking this one swim, we are done.
      break;
    }


  }
  $output = $output = '<h3>' . l(t('+ Add result'), 'swim/result/add') . '</h3>';
  $output .= theme('swim_meet', array('athlete' => $athlete, 'meet' => $meet, 'swims' => $swims));

  return $output;
}


/*
 * Theme function for a single meet results list.
 */
function theme_swim_meet($variables) {
  global $_SWIM_STROKES, $_SWIM_SEXES;

  $athlete = $variables['athlete'];
  $meet = $variables['meet'];
  $swims = $variables['swims'];
  $output = '';

  drupal_add_css(drupal_get_path('module', 'swim') . '/swim.css');
  drupal_set_title($meet->meet_name . ', ' . format_date($meet->start_date, 'custom', 'm/d/Y'));

  $header = array(
    array('data' => t('Swimmer'), 'field' => 'swim_swimmer'),
    array('data' => t('Event'), 'field' => 'swim_event', 'sort' => 'asc'),
    array('data' => t('Time')),
    array('data' => t('Place')),
    array('data' => t('Points')),
    array('data' => t('Best / Time Code'))
  );

  $rows = array();
  $swims_per_page = 50;
  $pager_total = count($swims);
  $page = pager_default_initialize($pager_total, $swims_per_page);
  $swim_counter = -1;
  foreach ($swims as $mid => $swim) {
    $swim_counter++;
    $query_page = (int) ($swim_counter / $swims_per_page);
    if ($query_page != $page) {
      continue;
    }

    $row = array('data' => array());

    if ($athlete == NULL) {
      $row['data'][] = array('data' => l(swim_athlete_format_full_name($swim), 'swim/athlete_list/' . $swim->aid, array('attributes' => array('title' => t('Swimmer\'s best times chart')))));
    }
    else {
      $row['data'][] = swim_athlete_format_full_name($swim);
    }

    $event_name = swim_event_get_full_name($swim);
    $row['data'][] = array('data' => l($event_name, 'swim/swimmer_event_history/' . $swim->aid . '/' . $swim->eid, array('attributes' => array('title' => t('Swimmer\'s event history')))));

    $row['data'][] = array('data' =>
      l($swim->time_code != '' ? '--' : swim_format_time($swim->time), 'swim/result/' . $swim->rid . '/edit'),
      'class' => 'time-col'
    );

    $row['data'][] = array('data' => $swim->place, 'class' => 'place-col');
    $row['data'][] = array('data' => $swim->points, 'class' => 'points-col');
    $att_data = '';
    if ($swim->time_code != '') {
      $att_data .= $swim->time_code . ' ';
    }
    if ($swim->pb > 0 && $swim->time_code == '') {
      $att_data .= t('BEST');
    }
    $row['data'][] = array('data' => $att_data, 'class' => 'att-col');
    $rows[] = $row;
  }

  $output = '';
  if ($athlete != NULL) {
    $output = '<h3>' . l(t('+ Show all athletes'), 'swim/meet_list/' . $meet->mid) . '</h3>';
  }
  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'caption' => t('Meet log'), 'sticky' => TRUE, 'attributes' => array('class' => array('meet-log'))));
  $output .= theme('pager', array('quantity' => $pager_total));
  return $output;
}

/*
 * List of all linked swimmers.
 */
function swim_my_swimmers_page() {
  // Don't want a title on this page.
  drupal_set_title('');
  // Don't want breadcrumbs on this page.
  drupal_set_breadcrumb(array());

  $build['swim_my_swimmers'] = drupal_get_form('swim_my_swimmers_form');
  return $build;
}

function swim_my_swimmers_form() {
  global $user;
  $form = array();
  $athletes = swim_get_athletes(array('status' => 1, 'linked_uid' => $user->uid), array());
  if (count($athletes) == 0) {
    $form['no_athletes'] = array('#markup' => t('No linked swimmers.'));
    return $form;
  }

  $form['swimmers'] = array();
  foreach ($athletes as $aid => $athlete) {
    $full_name_link = swim_athlete_format_full_name($athlete);
    if (variable_get('swim_results', 0) == 1) {
      $full_name_link = l($full_name_link, 'swim/athlete_list/' . $athlete->aid);
    }
    $form['swimmers'][$aid] = array(
      'ath_name' => array('#markup' => $full_name_link),
      'birthdate' => array('#markup' => format_date($athlete->birthdate, 'custom', 'm/d/Y') ),
    );
  }
  return $form;
}

function theme_swim_my_swimmers_form($variables) {
  drupal_add_css(drupal_get_path('module', 'swim') . '/swim.css');

  $form = $variables['form'];
  if (isset($form['no_athletes'])) {
    return drupal_render_children($form['no_athletes']);
  }
  $header = array(t('Swimmer'), t('Birth date'));
  $rows = array();
  foreach (element_children($form['swimmers']) as $aid) {
    $row = array('data' => array());
    $row['data'][] = drupal_render($form['swimmers'][$aid]['ath_name']);
    $row['data'][] = drupal_render($form['swimmers'][$aid]['birthdate']);
    $rows[] = $row;
  }

  $output = theme('table', array('header' => $header, 'rows' => $rows, 'sticky' => TRUE, 'attributes' => array('class' => array('athletes'))));
  $output .= drupal_render_children($form);
  return $output;
}


/*
 * List of all athletes with links to their summary pages.
 */
function swim_dashboard_swimmers_page() {
  // Don't want a title on this page.
  drupal_set_title('');
  // Don't want breadcrumbs on this page.
  drupal_set_breadcrumb(array());

  $variables = array('athletes' => swim_get_athletes(
    array('status' => 1),
    array(
      'order_by' => 'birthdate',
      'by_age_group' => 1
    )
  ));
  return theme('swim_dashboard_swimmers', $variables);
}
function theme_swim_dashboard_swimmers($variables) {
  drupal_add_css(drupal_get_path('module', 'swim') . '/swim.css');
  $athletes = $variables['athletes'];
  $header = array(t('Age Group'), t('Girls'), t('Boys'));
  $rows = array();
  foreach ($athletes as $age_group => $list) {
    $row = array('data' => array());
    $row['data'][] = $age_group;
    $boys = array();
    $girls = array();
    foreach ($list as $aid => $athlete) {
      if ($athlete->sex == 'M') {
        if (variable_get('swim_results', 0) == 1) {
          $boys[] = l($athlete->f_name . ' ' . $athlete->l_name, 'swim/athlete_list/' . $aid);
        }
        else {
          $boys[] = $athlete->f_name . ' ' . $athlete->l_name;
        }
      }
      else {
        if (variable_get('swim_results', 0) == 1) {
          $girls[] = l($athlete->f_name . ' ' . $athlete->l_name, 'swim/athlete_list/' . $aid);
        }
        else {
          $girls[] = $athlete->f_name . ' ' . $athlete->l_name;
        }
      }
    }
    $row['data'][] = implode('<br/>', $girls);
    $row['data'][] = implode('<br/>', $boys);
    $rows[] = $row;
  }

  $output = '';
  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'sticky' => TRUE, 'attributes' => array('class' => array('athletes'))));
  return $output;
}


/*
 * Single athlete's dashboard view with best times table and paged
 * meet list.
 */
function swim_athlete_page($athlete=NULL) {
  $variables = array(
    'athlete' => $athlete,
    'distances' => swim_get_distances(array('aid' => $athlete->aid)),
    'swims' => swim_get_best_swims(array('aid' => $athlete->aid)),
    'meets' => swim_get_meets_swims(array('aid' => $athlete->aid), array('by_season' => 1)),
  );
  return theme('swim_athlete', $variables);
}


/*
 * Theme function for the single athlete's dashboard page.
 */
function theme_swim_athlete($variables) {
  drupal_set_title(swim_athlete_format_full_name($variables['athlete']));
  drupal_add_css(drupal_get_path('module', 'swim') . '/swim.css');
  $output = '<div class="swim-dashboard">';
  $output .= theme('swim_best_times', $variables);
  $output .= theme('swim_athletes_meets', $variables);
  $output .= '</div>';
  return $output;
}


/*
 * Theme function for the best times table.
 */
function theme_swim_best_times($variables) {
  global $_SWIM_STROKES, $_SWIM_COURSES;
  $athlete = $variables['athlete'];
  $distances = $variables['distances'];
  $swims = $variables['swims'];

  $header = array();
  $header[] = array('data' => '&nbsp;', 'class' => 'odd');
  foreach ($distances as $dist) {
    $header[] = array('data' => $dist->distance, 'class' => 'distance');
  }

  $rows = array();
  foreach ($_SWIM_STROKES as $stroke => $stroke_info) {
    $row = array('data'=>array(), 'no_striping' => 1);
    $row['data'][] = array('data' => $stroke_info['name'], 'header' => 1, 'class' => 'stroke');
    foreach ($distances as $dist) {
      $cell_data = '';
      $got_one = 0;
      foreach ($_SWIM_COURSES as $course => $course_info) {
        if (isset($swims[$course][$dist->distance][$stroke])) {
          if ($got_one > 0) {
            $cell_data .= '<br/>';
          }
          if ($swims[$course][$dist->distance][$stroke]->time_code == '') {
            // If event has default course, don't print the course.
            if (variable_get('swim_default_course', 'SCY') == $swims[$course][$dist->distance][$stroke]->course) {
              $cell_data .= l(swim_format_time($swims[$course][$dist->distance][$stroke]->time), 'swim/swimmer_event_history/' . $athlete->aid . '/' . $swims[$course][$dist->distance][$stroke]->eid, array('attributes' => array('title' => $swims[$course][$dist->distance][$stroke]->meet_name . ', ' . $swims[$course][$dist->distance][$stroke]->season_name . '(' . $swims[$course][$dist->distance][$stroke]->course . ')')));
            }
            else {
              $cell_data .= l(swim_format_time($swims[$course][$dist->distance][$stroke]->time) . ' (' . $swims[$course][$dist->distance][$stroke]->course . ')', 'swim/swimmer_event_history/' . $athlete->aid . '/' . $swims[$course][$dist->distance][$stroke]->eid, array('attributes' => array('title' => $swims[$course][$dist->distance][$stroke]->meet_name . ', ' . $swims[$course][$dist->distance][$stroke]->season_name . '(' . $swims[$course][$dist->distance][$stroke]->course . ')')));
            }
          }
          else {
            if (variable_get('swim_default_course', 'SCY') == $swims[$course][$dist->distance][$stroke]->course) {
              $cell_data .= l($swims[$course][$dist->distance][$stroke]->time_code, 'swim/swimmer_event_history/' . $athlete->aid . '/' . $swims[$course][$dist->distance][$stroke]->eid, array('attributes' => array('title' => $swims[$course][$dist->distance][$stroke]->meet_name . ', ' . $swims[$course][$dist->distance][$stroke]->season_name)));
            }
            else {
              $cell_data .= l($swims[$course][$dist->distance][$stroke]->time_code . ' (' . $swims[$course][$dist->distance][$stroke]->course . ')', 'swim/swimmer_event_history/' . $athlete->aid . '/' . $swims[$course][$dist->distance][$stroke]->eid, array('attributes' => array('title' => $swims[$course][$dist->distance][$stroke]->meet_name . ', ' . $swims[$course][$dist->distance][$stroke]->season_name)));
            }
          }
          $got_one++;
        }
        else {
          $cell_data .= '&nbsp;';
        }
      }
      $row['data'][] = array('data' => $cell_data, 'class' => 'swim-time');
    }
    $rows[] = $row;
  }

  $output = '<h3>' . l(t('+ Add result'), 'swim/result/add/' . $athlete->aid) . '</h3>';
  $output .= '<div class="personal-best">';
  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'caption' => t('Best times'), 'sticky' => TRUE, 'attributes' => array('class' => array('best-times'))));
  return $output;
}


/*
 * Theme function for the meets that the athlete has participated in.
 */
function theme_swim_athletes_meets($variables) {
  $athlete = $variables['athlete'];
  $meets = $variables['meets'];
  $pager_total = count($meets);
  $page = pager_default_initialize($pager_total, 1, $element = 0);
  $query_page = -1;

  $header = array(t('Date'), t('Name'), t('Swims'), t('Points'), t('Bests'));
  $rows = array();
  $displayed_seasson = '';
  foreach ($meets as $season => $season_meets) {
    $query_page++;
    if ($query_page != $page) {
      continue;
    }
    $displayed_seasson = filter_xss($season);
    foreach ($season_meets as $mid => $meet) {
      $row = array('data' => array());
      $row['data'][] = format_date($meet->start_date, 'custom', 'Y-m-d');
      $row['data'][] = l($meet->meet_name, 'swim/meet_list/' . $mid . '/' . $athlete->aid);
      $row['data'][] = $meet->swims;
      $row['data'][] = (isset($meet->points) ? $meet->points : '');
      $row['data'][] = (isset($meet->pb) ? $meet->pb : '');
      $rows[] = $row;
    }
  }
  $output = theme('table', array('header' => $header, 'rows' => $rows, 'caption' => $displayed_seasson . t(' Meets'), 'sticky' => TRUE, 'attributes' => array('class' => array('meet-swims'))));
  $output .= theme('pager', array('quantity' => $pager_total));
  return $output;
}


/*
 * Athlete's history for an event.
 */
function swim_event_history_page($athlete, $event) {
  $variables = array(
    'athlete' => $athlete,
    'event' => $event,
    'swims' => swim_get_swims(array('aid' => $athlete->aid, 'stroke' => $event->stroke, 'distance' => $event->distance, 'course' => $event->course)),
    'best_swims' => swim_get_swims(array('aid' => $athlete->aid, 'best_swims' => 1, 'stroke' => $event->stroke, 'distance' => $event->distance, 'course' => $event->course)),
    'records' => swim_get_records(array('eid' => $event->eid)),
  );
  return theme('swim_event_history', $variables);
}


/*
 * Theme function for the athlete's event history.
 */
function theme_swim_event_history($variables) {
  global $_SWIM_STROKES, $_SWIM_SEXES;

  $athlete = $variables['athlete'];
  $event = $variables['event'];
  $swims = $variables['swims'];
  $best_swims = $variables['best_swims'];
  $records = $variables['records'];

  drupal_add_css(drupal_get_path('module', 'swim') . '/swim.css');
  $page_title = '';
  if (variable_get('swim_default_course', 'SCY') == $event->course) {
    $page_title = swim_athlete_format_full_name($athlete) . ', ' . $_SWIM_SEXES[$event->sex] . ' ' . $event->distance . ' ' . $_SWIM_STROKES[$event->stroke]['long'];
  }
  else {
    $page_title = swim_athlete_format_full_name($athlete) . ', ' . $_SWIM_SEXES[$event->sex] . ' ' . $event->distance . ' ' . $_SWIM_STROKES[$event->stroke]['long'] . ', ' . $event->course;
  }
  drupal_set_title($page_title);

  $header = array(t('Meet date'), t('Meet name'), t('Age Group'), t('Course'), t('Time'), t('Diff'), t('Place'), t('Points'));
  $rows = array();
  $last_season = '';

  $g_chart = new stdClass;
  $g_chart->data = array();
  foreach ($swims as $mid => $swim) {
    $chart_data_row = array();
    $formatted_date = format_date($swim->start_date, 'custom', 'Y-m-d');
    $formatted_time = swim_format_time($swim->time);

    $class = '';
    if (!empty($last_season) && $last_season != $swim->season_name) {
      $class = ' table-break';
    }

    $row = array('data' => array());
    $row['data'][] = array('data' => $formatted_date, 'class' => "meet-date$class");
    $row['data'][] = array('data' => $swim->meet_name, 'class' => "meet-name$class");
    $row['data'][] = array('data' => swim_event_get_age_group($swim), 'class' => "age-group$class");
    $row['data'][] = array('data' => $swim->course, 'class' => "course$class");
    if ($swim->time_code != '') {
      $row['data'][] = array('data' => $swim->time_code, 'class' => "time-col$class");
      $row['data'][] = array('data' => '&nbsp;', 'class' => "time-col$class");
    }
    else {
      $best_time;
      foreach ($best_swims as $rid => $b_swim) {
        // Skip best swim if meet has not yet happened.
        if ($b_swim->start_date > $swim->start_date) {
          continue;
        }

        // This is the best meet.
        // Don't end the loop though, get the last best for a time diff.
        if ($b_swim->start_date == $swim->start_date) {
          $best_time = $b_swim->time;
        }

        // This is not the best meet.  End the loop.
        // This was the last best though.
        else {
          if (isset($best_time)) {
            $row['data'][] = array('data' => '*' . $formatted_time, 'class' => "time-col personal-best$class", 'title' => t('Personal Best'));
            $row['data'][] = array('data' => '(' . swim_format_time($best_time - $b_swim->time) . ')', 'class' => "time-col$class");
            $chart_data_row = array($formatted_date, $swim->time / 100.0);
            foreach ($records as $rec_id => $record) {
              $chart_data_row[] = $record->time / 100.0;
            }

            unset($best_time);
          }
          else {
            $row['data'][] = array('data' => $formatted_time, 'class' => "time-col$class");
            $row['data'][] = array('data' => swim_format_time($swim->time - $b_swim->time), 'class' => "time-col$class");
            $chart_data_row = array($formatted_date, $swim->time / 100.0);
            foreach ($records as $rec_id => $record) {
              $chart_data_row[] = $record->time / 100.0;
            }

          }
          break;
        }
      }
      // Very last swim by sequence was a best time.
      if (isset($best_time)) {
        $row['data'][] = array('data' => '*' . $formatted_time, 'class' => "time-col personal-best$class", 'title' => t('Personal Best'));
        $row['data'][] = array('data' => '&nbsp;', 'class' => "time-col$class");
        $chart_data_row = array($formatted_date, $swim->time / 100.0);
        foreach ($records as $rec_id => $record) {
          $chart_data_row[] = $record->time / 100.0;
        }
        unset($best_time);
      }
    }


    $row['data'][] = array('data' => $swim->place, 'class' => "place-col$class");
    $row['data'][] = array('data' => $swim->points, 'class' => "points-col$class");

    $rows[] = $row;
    // Don't add swims without a valid time to chart. (ie DQ).
    if (isset($swim->time) && $swim->time != 0) {
      $g_chart->data[] = $chart_data_row;
    }
    $last_season = $swim->season_name;
  }

  $output = '';

  // Reverse sort and only keep the last 6 swims.
  $g_chart->data = array_reverse(array_slice($g_chart->data, 0, 6));
  // Add data titles after sort and crop.
  array_unshift($g_chart->data, array('Swim Dates', 'Time'));
  $series_options = array();
  foreach ($records as $rec_id => $record) {
    $g_chart->data[0][] = $record->record_name;
    $series_options[(int) (count($g_chart->data[0]) - 2)] = array('type' => 'line');
  }

  // Chart object.
  //$g_chart = new stdClass;
  $g_chart->visualization = 'ComboChart';
  $g_chart->packages = array('corechart');
  $g_chart->container_id = 'chart_div';


  $g_chart->options = array(
    'width' => 750,
    'height' => 450,
    'title' => 'Last 6 Swims',
    'hAxis' => array(
      'title' => 'Swim Dates',
      'titleTextStyle' => array(
        'color' => 'black'
      )
    ),
    'vAxis' => array(
      'title' => 'Seconds',
      'titleTextStyle' => array(
        'color' => 'black'
      ),
    ),
    'seriesType' => 'bars',
    'series' => $series_options
  );

  $gct_obj = new swim_google_chart_tools_helper();

  // Don't render the chart if there are only headers.
  if (count($g_chart->data) > 1) {
    $output .= $gct_obj->render($g_chart);
    $output .= '<div id="' . $g_chart->container_id . '"></div>';
  }

  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'caption' => t('Event history'), 'sticky' => TRUE, 'attributes' => array('class' => array('swim-log'))));
  return $output;
}
