<?php
/**
 * @file
 *
 * Provides the object definitions for the google chart tools helper class.
 */

/**
 * Definition of the google chart tools helper class.
 */
class swim_google_chart_tools_helper {

  // Properties
  private $props = array();

  // Magic overload method.
  public function __get($name) {
    if (array_key_exists($name, $this->props)) {
      return $this->props[$name];
    }
    trigger_error('Undefined property via __get(): ' . $name, E_USER_NOTICE);
    return NULL;
  }

  // Constructor
  public function __construct() {
    $this->props = array(
      'library_loaded' => FALSE,
    );
  }

  /**
   * Return a string with the appropriate javascript to render a chart.
   *
   * @param $chart
   *   A stdClass chart object.
   * @return
   *   A string consisting of JavaScript to embed into a web page.
   */
  function render($chart) {
    if (!isset($chart)) {
      $chart = new stdClass;
      $chart->visualization = 'ColumnChart';
      $chart->packages = array('corechart');
      $chart->container_id = 'chart_div';
      $chart->data = array(
        array('Year', 'Sales', 'Expenses'),
        array('2004', 1000, 400),
        array('2005', 1170, 460),
        array('2006', 660, 1120),
        array('2007', 1030, 540),
      );

      $chart->options = array(
        'width' => 400,
        'height' => 240,
        'title' => 'Company Performance',
        'hAxis' => array(
          'title' => 'Year',
          'titleTextStyle' => array(
            'color' => 'red'
          )
        ),
        'seriesType' => 'bars',
        'series' => array(1 => array('type' => 'line')),
      );
    }

    $packages = json_encode($chart->packages);
    $data = json_encode($chart->data);
    $options = json_encode($chart->options);
    
    // Replace any quoted pure numeric keys.  
    // Bug in php, fixed in php 5.3.3.
    // Remove once Drupal requires php >= 5.3.3
    $options = preg_replace('/"(\d+)":/', '\1:', $options); 
    
    $chart_output = '';
    if (!$this->props['library_loaded']) {
      $chart_output .= '<script type="text/javascript" src="https://www.google.com/jsapi"></script>';
      $this->props['library_loaded'] = TRUE;
    }
    $chart_output .= <<<EOD
    <script type="text/javascript">
      google.load('visualization', '1', {packages: $packages});
    </script>
    <script type="text/javascript">
      function drawVisualization() {
        var data = google.visualization.arrayToDataTable($data);
        var options = $options;
        var chart = new google.visualization.$chart->visualization(document.getElementById('$chart->container_id'));
        chart.draw(data, options);
      }
      google.setOnLoadCallback(drawVisualization);
    </script>
EOD;



    return $chart_output;
  }
}
